/*
 * ni-iface-helper.c
 * This file is part of Network Inador
 *
 * Copyright (C) 2021 - Félix Arreola Rodríguez
 *
 * Network Inador is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Network Inador is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Network Inador; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

#include <stdint.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "network-inador-manager.h"

#define COMMAND_SOCKET_PATH "/tmp/network-inador.socket"

extern char** environ;

int debug (int argc, char *argv[]) {
	char **item;
	int fd;
	
	fd = open ("/tmp/var_dhcp.env", O_CREAT | O_WRONLY | O_TRUNC, 0644);
	
	if (fd < 0) {
		return -1;
	}
	
	for (item = environ; *item; item++) {
		char *name, *val, **p;

		/* Split on the = */
		name = strdup (*item);
		val = strchr (name, '=');
		if (!val || val == name) {
			free (name);
			continue;
		}
		*val++ = '\0';

		/* Ignore non-DCHP-related environment variables
		for (p = (char **) ignore; *p; p++) {
			if (strncmp (name, *p, strlen (*p)) == 0)
				goto next;
		}*/
		
		write (fd, "Name: <", 7);
		write (fd, name, strlen (name));
		write (fd, ">, Value: <", 11);
		write (fd, val, strlen (val));
		write (fd, ">\n", 2);
	}
	
	close (fd);
	
	return 0;
}

char *retrieve_env (const char *name) {
	char *bus, *dup, *get;
	
	get = getenv (name);
	
	if (get == NULL) {
		return NULL;
	}
	
	bus = strchr (get, '=');
	
	if (!bus || bus == get) {
		return NULL;
	}
	
	dup = strdup (&get[1]);
	
	if (dup == NULL) {
		return NULL;
	}
	
	return dup;
}

int wait_for_ack_or_error (int s) {
	unsigned char buffer[128];
	int ret;
	
	ret = recv (s, buffer, sizeof (buffer), 0);
	
	if (ret < 3) {
		return -1;
	}
	
	if (buffer[0] == NET_INADOR_TYPE_RESPONSE_ERROR) {
		return -1;
	}
	
	if (buffer[0] == NET_INADOR_TYPE_RESPONSE && buffer[1] == NET_INADOR_RESPONSE_EXECUTED) {
		return 0;
	}
	
	return -1;
}

void send_preinit (int s, char *interface) {
	unsigned char buffer[128];
	
	buffer[0] = NET_INADOR_TYPE_COMMAND;
	buffer[1] = NET_INADOR_COMMAND_DHCP_SET_STATUS;
	
	buffer[2] = NET_INADOR_DHCP_STATUS_INITING;
	buffer[3] = strlen (interface);
	strncpy (&buffer[4], interface, buffer[3]);
	
	send (s, buffer, 3 + buffer[3], 0);
	
	if (wait_for_ack_or_error (s) < 0) {
		close (s);
		return;
	}
	
	close (s);
}

int main (int argc, char *argv[]) {
	char *reason, *interface;
	int s, ret;
	struct sockaddr_un path_dest;
	
	debug (argc, argv);
	
	reason = retrieve_env ("reason");
	
	interface = retrieve_env ("interface");
	
	if (reason == NULL || interface == NULL) {
		return 1;
	}
	
	/* Intentar abrir el socket a la escucha */
	s = socket (AF_UNIX, SOCK_SEQPACKET | SOCK_CLOEXEC, 0);
	
	if (s < 0) {
		return 1;
	}
	
	path_dest.sun_family = AF_UNIX;
	strncpy (path_dest.sun_path, COMMAND_SOCKET_PATH, sizeof (path_dest.sun_path));
	ret = connect (s, (struct sockaddr *) &path_dest, sizeof (path_dest));
	
	if (ret < 0) {
		perror ("Connect");
		close (s);
		return 1;
	}
	
	if (strcmp (reason, "PREINIT") == 0 || strcmp (reason, "FAIL") == 0) {
		send_preinit (s, interface);
		
		return 0;
	} else if (strcmp (reason, "REBOOT") == 0 || strcmp (reason, "RENEW") == 0) {
		/*
		 * Los valores más importantes recibidos:
		 * new_domain_name_servers -> los DNS
		 * new_expiry -> el timestamp de la hora
		 * new_domain_name -> DNS search name
		 * new_routers -> Lista de routers
		 * new_subnet_mask -> Máscara de red en formato largo
		 * new_broadcast_address -> Dirección broadcast
		 * new_ip_address -> Dirección IP
		 * new_dhcp_lease_time -> Tiempo en segundos
		 */
	} else if (strcmp (reason, "EXPIRE") == 0) {
		/* Los mismos valores, pero con old_ */
	} else {
		
		
		close (s);
	}
	
	return 0;
}
